//
// FMCU Visualization code
//
// by Fran Sanchez (The Beach Lab)
// Do what the F you want with it Licence 
// Jan 2023

// CSS
import './style.css'

// Modules
import * as mqtt from 'mqtt/dist/mqtt.min';  // Vite support
import * as THREE from 'three'
import ThreeGlobe from 'three-globe'
import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls.js'
import { Lensflare, LensflareElement } from 'three/examples/jsm/objects/Lensflare.js';
import * as utils from './utils.js'

// Start loading stars
const stars = []
fetch('/files/stars_clean.json')
.then(response => response.json())
.then(result => { 
      stars.push(...result)
      drawStars(stars,myScene)
    })

// Listen for MQTT update
const url = 'wss://mosquitto.beachlab.org:8083/'
// Create an MQTT client instance
const options = {
  // Clean session
  clean: true,
  connectTimeout: 4000,
  // Authentication
  username: 'fmcu',
  password: 'Push 1 Button',
}
const client  = mqtt.connect(url, options)
client.on('connect', function () {
  console.log('Connected to ws')
  // Subscribe to a online
  client.subscribe('fmcu/online', function (err) {
    if (!err) {
      // subscribed OK
      console.log('Subscribed to fmcu/online')
    }
  })
  client.subscribe('fmcu/sun', function (err) {
    if (!err) {
      // subscribed OK
      console.log('Subscribed to fmcu/sun')
    }
  })

})
// Receive messages
client.on('message', function (topic, message) {
  // message is Buffer
  const msg = message.toString()
  if (topic == 'fmcu/sun' && msg == 'true') {
      // get new sun coordinates
      [xsun, ysun, zsun] = utils.getSunCoords()
      // move the sun
      light.position.set(xsun, ysun, zsun);  
      console.log('Sun moved',Date.now())  
  } else if (topic == 'fmcu/online') {
      // store temporarily for some checks
      const temp = JSON.parse(msg)
      // clear old data
      id_online = []
      for (let i in temp) {
        // ignore if the lab is not in the list
        if (pointsData.find(x => x.id == parseInt(temp[i].id)) != undefined) {
          id_online.push(temp[i])
        }
      }
      if (id_online.length == 0) {
        document.getElementById('online').innerHTML ="who will push the button?";
        document.getElementById('online').setAttribute("class","blink_me")
        document.getElementById('lab_count').style.visibility ="hidden";
      } else {
        document.getElementById('online').innerHTML = id_online.length + " online";
        document.getElementById('online').removeAttribute("class","blink_me")
        document.getElementById('lab_count').style.visibility ="visible";
      }
    // recalculate
      calculate_arcs()
      calculate_rings()
      myGlobe.arcsData(arcsData)
      myGlobe.ringsData(ringsData)
  }
  
})

// get a random quote every time the page is reloaded
const randomText = ["It's automatic, but you<br>have to press the button","Every important machine<br>has a big red button","Maybe you should just<br>press the button","You push the button,<br>we do the rest","The instructions say:<br>Press the button","Frankenstein Multipoint<br>Conference Unit","All we do for a living<br>is press a button","Some people know<br>which buttons to press","Hit the F button<br>Let us know you're online","Whatever you do,<br>don't push this button","The most over-engineered<br>button since Cold War"]
const text = document.getElementById('tag')
function getRandomInt(max) {
  return Math.floor(Math.random() * max);
}
text.innerHTML = randomText[getRandomInt(randomText.length)]

// state vars
var rotating = true

// general vars
const arcColors = ['yellow', 'red', 'yellow'] // color gradient
const arcColorsFA = ['yellow', 'red', 'yellow'] // color gradient for Fab Academy nodes
const mypointColor = ['red', 'white', 'blue', 'green'][Math.round(Math.random() * 3)]
const colorInterpolator = t => `rgba(0,255,255,${1-0.5*t})`; // RGB and Alpha fades out with t


// init data variables
var id_online = [] // array of labs online
const labs = []
var pointsData = []
var ringsData = []
var arcsData = []

// 1st time read online labs json
fetch('/files/labs_online.json')
    .then(response => response.json())
    .then(result => {
      id_online.push(...result)
      if (id_online.length == 0) {
        document.getElementById('online').innerHTML ="someone push the button™";
        document.getElementById('online').setAttribute("class","blink_me")
        document.getElementById('lab_count').style.visibility ="hidden";

      } else {
        document.getElementById('online').innerHTML = id_online.length + " online";
      }
    // console.log(id_online)
      // now read labs and initial globe draw
      read_labs()
    });

// read clean labs json
function read_labs() {
fetch('/files/labs_clean.json')
    .then(response => response.json())
    .then(result => {
      labs.push(...result)
      document.getElementById('lab_count').innerHTML = labs.length + " fab labs";
      //console.log(labs)
      // calculate data for globe
      calculate_labpoints()
      calculate_rings()
      calculate_arcs()
      myGlobe
        // draw labs
        .pointsData(pointsData)
        .pointAltitude(0.0025)
        .pointRadius(0.07)
        // .pointColor('mypointColor')
        // draw rings
        .ringsData(ringsData)
        .ringColor(() => colorInterpolator)
        .ringMaxRadius(5)
        .ringPropagationSpeed(0.8)
        .ringRepeatPeriod(2000)
        // draw arcs
        .arcsData(arcsData)
        .arcAltitudeAutoScale(0.25)
        //.arcAltitude(0)
        .arcColor('color')
        .arcDashLength(1)
        .arcDashGap(0)
        .arcDashInitialGap(0)
        // .arcDashAnimateTime(20000)
        .arcStroke(0.15)
        .arcsTransitionDuration(1000)

    });
  }

// build labs array with geolocation data
function calculate_labpoints() {
for (let i in labs) {
  const obj = {};
  obj["id"] = labs[i].id
  obj["name"] = labs[i].name
  obj["lat"] = labs[i].latitude
  obj["lng"] = labs[i].longitude
  // obj["color"] = mypointColor
  // discard labs without geocoding info
  if (obj["lat"] != null && obj["lng"] != null){
    pointsData.push(obj);
  }
}
//  console.log(pointsData);
}



// Rings Data
function calculate_rings() {
  ringsData = [] // reset rings
  for (let i in id_online) {
    const obj = {};
    obj["id"] = id_online[i].id
    obj["name"] = pointsData.find(x => x.id === parseInt(id_online[i].id)).name 
    obj["lat"] = pointsData.find(x => x.id === parseInt(id_online[i].id)).lat
    obj["lng"] = pointsData.find(x => x.id === parseInt(id_online[i].id)).lng
    ringsData.push(obj);
  }
//  console.log(ringsData);
}

// set arcs FMCU
function calculate_arcs() {
  arcsData = [] // reset arcs
  for (var i = 0; i < id_online.length; i++) {
    for (var j = i+1; j < id_online.length; j++) {
      const obj = {};
      obj["origin"] = pointsData.find(x => x.id === parseInt(id_online[i].id)).name
      obj["startLat"] = pointsData.find(x => x.id === parseInt(id_online[i].id)).lat 
      obj["startLng"] = pointsData.find(x => x.id === parseInt(id_online[i].id)).lng
      obj["destination"] = pointsData.find(x => x.id === parseInt(id_online[j].id)).name
      obj["endLat"] = pointsData.find(x => x.id === parseInt(id_online[j].id)).lat
      obj["endLng"] = pointsData.find(x => x.id === parseInt(id_online[j].id)).lng
      obj["color"] = arcColors
      // console.log(i,j)
      arcsData.push(obj);
    }
  }
  // console.log(arcsData)
}

// set arcs Fab Academy 229 MIT 1155 is bold? 13 is odc?
// const id_fa = [14,1841,2386,875,1155,116,236,841,126,1099,409,1568,10,2398,143,428,142,2258,13,157,62,1487,266,896,36,2121,1616,499,770,226,325,1214,1492,355,1157,793,1363,2377,1256,767,1542,238,2529,150,72,1899,109,89,469,13,552,292,1449,689,1205,2191,786,893,176,268,1587,21,857,40,516,2265]
// const arcsfaData = []
// for (let i in id_fa) {
//   // check if the fab academy lab in the pointsData array
//   if (pointsData.find(x => x.id == id_fa[i]) != undefined) { 
//     const obj = {};
//     obj["origin"] = pointsData.find(x => x.id === 229).name
//     obj["startLat"] = pointsData.find(x => x.id === 229).lat 
//     obj["startLng"] = pointsData.find(x => x.id === 229).lng
//     obj["destination"] = pointsData.find(x => x.id === id_fa[i]).name
//     obj["endLat"] = pointsData.find(x => x.id === id_fa[i]).lat
//     obj["endLng"] = pointsData.find(x => x.id === id_fa[i]).lng
//     obj["color"] = arcColorsFA
//     arcsfaData.push(obj);
//   }
// }

// Define Globe
const myGlobe = new ThreeGlobe()
  .globeImageUrl('earth-etopov2.webp')
  .bumpImageUrl('earth-relief-2k.webp')
  .showGraticules(false)


// Setup renderer
const renderer = new THREE.WebGLRenderer({antialiasing:true});
renderer.setSize(window.innerWidth, window.innerHeight);
renderer.setPixelRatio( window.devicePixelRatio * 1.0 );
document.getElementById('app').appendChild(renderer.domElement);



// get sun coords first time you load page
var [xsun, ysun, zsun] = utils.getSunCoords()

// Brightests stars
// catalog from https://github.com/aduboisforge/Bright-Star-Catalog-JSON

function drawStars(stars,scene) {
  const distance = 1200;
  const particlesGeometry = new THREE.BufferGeometry();
  const positions = new Float32Array(stars.length * 3);
  const colors = new Float32Array(stars.length * 3);
  const sizes = new Float32Array(stars.length);

  for (let i = 0; i < stars.length; i++) {
    positions[i * 3] = distance*stars[i].x;
    positions[i * 3 + 1] = distance*stars[i].y;
    positions[i * 3 + 2] = distance*stars[i].z;
    colors[i * 3] = stars[i].color.r / 255;
    colors[i * 3 + 1] = stars[i].color.g / 255;
    colors[i * 3 + 2] = stars[i].color.b / 255;
    sizes[i] = stars[i].size;
  }

  // console.log(positions,colors,sizes)

  particlesGeometry.setAttribute('position', new THREE.BufferAttribute(positions, 3)) // Create the Three.js BufferAttribute and specify that each information is composed of 3 values
  particlesGeometry.setAttribute('color', new THREE.BufferAttribute(colors, 3))
  particlesGeometry.setAttribute("size", new THREE.BufferAttribute(sizes, 1));
  
  const vertexShader = `
    uniform float size;
    uniform vec3 color;
    varying vec3 vColor;
    void main() {
      vColor = color;
      vec4 mvPosition = modelViewMatrix * vec4(position, 1.0);
      gl_PointSize = size * (1200.0 / -mvPosition.z);
      gl_Position = projectionMatrix * mvPosition;
    }
  `;

  const fragmentShader = `
    varying vec3 vColor;
    void main() {
      gl_FragColor = vec4(vColor, 1.0);
    }
  `;

  const particlesMaterial = new THREE.ShaderMaterial({
    uniforms: {
      size: { value: 2 },
      color: { value: new THREE.Color(0xffffff) }
    },
    vertexShader: vertexShader,
    fragmentShader: fragmentShader,
    transparent: true
  });

  const starParticles = new THREE.Points(particlesGeometry, particlesMaterial);
  scene.add(starParticles);
  }



// Setup scene
const myScene = new THREE.Scene();
myScene.add(myGlobe);
myScene.add(new THREE.AmbientLight(0x353535));
const light = new THREE.DirectionalLight(0xffffd6, 0.75);
light.position.set(xsun, ysun, zsun);
myScene.add(light);


function addLight( h, s, l, x, y, z ) {


  const lensflare = new Lensflare();
  lensflare.addElement( new LensflareElement( textureFlare0, 700, 0 ) );
  lensflare.addElement( new LensflareElement( textureFlare3, 60, 0.6 ) );
  lensflare.addElement( new LensflareElement( textureFlare3, 70, 0.7 ) );
  lensflare.addElement( new LensflareElement( textureFlare3, 120, 0.9 ) );
  lensflare.addElement( new LensflareElement( textureFlare3, 70, 1 ) );
  light.add( lensflare );

}

// lensflares
const textureLoader = new THREE.TextureLoader();
const textureFlare0 = textureLoader.load( '/lensflare0.png' );
const textureFlare3 = textureLoader.load( '/lensflare3.png' );
// addLight( 0.55, 0.9, 0.5, 5000, 0, - 1000 );
addLight( 0.08, 0.8, 0.5, 0, 0, - 1000 );
// addLight( 0.995, 0.5, 0.9, 5000, 5000, - 1000 );



// Setup camera
const camera = new THREE.PerspectiveCamera();
camera.aspect = window.innerWidth/window.innerHeight;
camera.updateProjectionMatrix();
camera.position.z = 300;

// // Add camera controls
const tbControls = new OrbitControls(camera, renderer.domElement);
tbControls.minDistance = 130;
tbControls.maxDistance = 500;
tbControls.autoRotate = true;
tbControls.autoRotateSpeed = 0.25;
tbControls.enablePan = true;
tbControls.enableDamping = true;
tbControls.dampingFactor = 0.05
tbControls.rotateSpeed = 0.5;
tbControls.zoomSpeed = 0.2;

// Kick-off renderer
(function animate() { // IIFE
  // Frame cycle
  tbControls.update();
  renderer.render(myScene, camera);
  requestAnimationFrame(animate);
})();

// Redraw on resize
window.addEventListener( 'resize', onWindowResize );
function onWindowResize() {
  camera.aspect = window.innerWidth / window.innerHeight;
  camera.updateProjectionMatrix();
  renderer.setSize( window.innerWidth, window.innerHeight );
}

// Key Controls. Toggle Earth Rotation
window.addEventListener( 'keydown', (event) => {
  if (event.key == 'r') {
    rotating ? tbControls.autoRotate = false : tbControls.autoRotate = true;
    rotating ? rotating = false : rotating = true;
  }
})
