# FMCU with Zoom Meeting integration

![FMCU Zoom integration](fmcu_zoom_integration.jpg)

Francisco Sanchez Arroyo and Þórarinn Bjartur Breiðfjörð Gunnarsson started the [Frankenstein MCU project](http://academany.fabcloud.io/fabacademy/2023/instructors-bootcamp/Projects/FrankensteinMCU/) at the 2023 Fab Academy Instructor's bootcamp in Amsterdam. Its goal is to connect all the Fab Labs together with a web interface where you can see which labs are online. Each lab fabricates a button in its own style to connect to the network. 

The original [Multipoint Conference Unit](http://wiki.fablab.is/wiki/Multipoint_Conference_Unit_(MCU)) ran on a server in a basement at the Center for Bits and Atoms MIT. That was long before video conferencing became widely used, but now we can use Zoom to easily connect hundreds of Fab Labs at the same time without setting up the streaming infrastructure ourselves. 

This repo is an attempt to integrate Zoom video streaming into the platform using the [Zoom Meeting SDK for web](https://developers.zoom.us/docs/meeting-sdk/web/). The code we're using is [this JavaScript example](https://github.com/zoom/meetingsdk-javascript-sample). It works, but we still need to change some things so that you don't have to sign up for a Zoom Developer account to connect!
## How to run locally

To run the FMCU-Zoom website locally, you need to follow these steps:

## Zoom Meeting SDK Authentication server setup
1.  Clone [meetingsdk-auth-endpoint-sample-master](https://gitlab.fabcloud.org/pub/project/bootcamp-2023/fmcu/-/tree/main/meetingsdk-auth-endpoint-sample-master) (or download the zip file)

![Zip file download](images/zip.jpg)
1.  Download [Node.js](https://nodejs.org/en/download/)
1.  Get [Visual Studio Code](https://code.visualstudio.com/)
1.  In VSCode, go to File -> Open Folder and choose the meetingsdk-auth-endpoint-sample-master folder
1. Open a Git Bash terminal with Terminal -> New Terminal and then use  the little down arrow at the bottom right of the screen to choose Git Bash 
![Git Bash](gitbash.jpg)
1.  Enter `npm install` to install dependencies
1.  Enter `touch .env` to create an environment file
1.  Add the following code to the `.env` file and insert your [Zoom Meeting SDK credentials](https://developers.zoom.us/docs/meeting-sdk/developer-accounts/#get-meeting-sdk-credentials):
    ```
        ZOOM_MEETING_SDK_KEY=MEETING_SDK_KEY_HERE
        ZOOM_MEETING_SDK_SECRET=MEETING_SDK_SECRET_HERE
    ```
1. Save and close `.env`.
1. Enter `npm run start` to start the authentication server.

## FMCU setup
1. Clone the [fmcu-zoom](https://gitlab.fabcloud.org/pub/project/bootcamp-2023/fmcu/-/tree/main/fmcu-zoom) repo  (or download the zip file)
1. Open a second instance of VSCode
1. In VSCode, go to File -> Open Folder and choose the fmcu-zoom folder
1. Open a Git Bash terminal
1. Enter `npm install` in the terminal to install dependencies
1. The [meetingsdk-sample-javascript](https://github.com/zoom/meetingsdk-javascript-sample) has been integrated into the fmcu code. All you need to do now to make the Zoom component work is to open [component-view.js](https://gitlab.fabcloud.org/pub/project/bootcamp-2023/fmcu/-/blob/main/fmcu-zoom/component-view.js) in VSCode and fill in these three fields in lines 6, 7 and 8:
    ```
    var sdkKey = ''
    var meetingNumber = ''
    var passWord = ''
    ```
    - You get your sdkKey in the [Zoom Developer portal](https://developers.zoom.us/docs/meeting-sdk/developer-accounts/#get-meeting-sdk-credentials). The zoom-readme.md file says that you can also use your Client ID, but that hasn't been tested yet.
    - You also need to input a meeting number to connect to (I've been connecting to the Fab Academy Zoom meeting, because it's always available)
    - If the meeting has a password, you put that in the passWord field.
1. Enter `npm run dev`, visit the website and now the Join Meeting button in the top right corner should work. The authentication server signs you in to the meeting when you press the button. Then you can turn on video and audio, move the Zoom component around and resize it.
1. To stop running the FMCU server, go to the terminal in in VSCode and hit Ctrl-C. The same goes for the authentication server.


## What was added to the FMCU repo for Zoom integration
- The images folder
- client-view.js (not used)
- component-view.js
- favicon.ico
- styles.css
- zoom_readme.md
- The following code was added to the middle of the body of index.html:
    ``` html
        <!-- For Component View -->
    <div id="meetingSDKElement"  class="repo">
      <!-- Zoom Meeting SDK Rendered Here -->
      <button onClick="getSignature()">Join Meeting</button>
    </div>
    ```
- The following code was added to the body of index.html, right at the end:
    ``` html
        <!-- For Component and Client View -->
    <script type="text/javascript" src="https://source.zoom.us/2.11.5/lib/vendor/react.min.js"></script>
    <script type="text/javascript" src="https://source.zoom.us/2.11.5/lib/vendor/react-dom.min.js"></script>
    <script type="text/javascript" src="https://source.zoom.us/2.11.5/lib/vendor/redux.min.js"></script>
    <script type="text/javascript" src="https://source.zoom.us/2.11.5/lib/vendor/redux-thunk.min.js"></script>
    <script type="text/javascript" src="https://source.zoom.us/2.11.5/lib/vendor/lodash.min.js"></script>
    
    <!-- For Component View -->
    <script type="text/javascript" src="https://source.zoom.us/2.11.5/zoom-meeting-embedded-2.11.5.min.js"></script>
    <script type="text/javascript" src="component-view.js"></script>
    ```

These files were taken directly from [meetingsdk-sample-javascript](https://github.com/zoom/meetingsdk-javascript-sample). The Zoom integration was an [Interface and Application Programming project](https://fabacademy.org/2023/labs/isafjordur/students/svavar-konradsson/assignments/week14.html) at [Fab Academy 2023](https://fabacademy.org/2023/).

## Next steps
Have a look at the readme in the [meetingsdk-auth-endpoint-sample-master](https://gitlab.fabcloud.org/pub/project/bootcamp-2023/fmcu/-/tree/main/meetingsdk-auth-endpoint-sample-master) folder. We might find something useful about logging in users in a production web site under the Usage and Deployment headings in the readme.

The [zoom_readme.md](zoom_readme.md) file in this directory is the original readme file for the Zoom Meeting SDK Javascript example.

To build the FMCU for deployment, have a look at the [fmcu_development.md](fmcu_development.md) file.
