// Catalog of the brightest starts in XYZ coordinates for FMCU
// by Fran Sanchez (The Beach Lab) and ChatGPT
// February 2023

const stars = require('./public/files/bsc.json') 
const { writeFile } = require('node:fs');

var stars_clean = []

for (let i=0; i<stars.length; i++) {
    const obj = {}
    // right ascension
    const ra_rad = rightAscensionToRadians(stars[i].RA)
    // declination
    const dec_rad = declinationToRadians(stars[i].DEC)
    //coordinates
    var z = Math.cos(ra_rad) * Math.cos(dec_rad) // projection to xy plane and projection to x axis
    var x = Math.sin(ra_rad) * Math.cos(dec_rad)
    var y = Math.sin(dec_rad)
    obj["x"] = x
    obj["z"] = z
    obj["y"] = y
    obj["size"] = mapMagnitude(parseFloat(stars[i].MAG))
    obj["class"] = stars[i]["Title HD"].slice(0,1).toUpperCase()
    obj["color"] =spectralClassToRGB(stars[i]["Title HD"].slice(0,1).toUpperCase())

    stars_clean.push(obj)
}

function rightAscensionToRadians(raString) {
    let { hours, minutes, seconds } = parseRightAscension(raString)
    const totalSeconds = 3600 * hours + 60 * minutes + seconds;
    const totalHours = totalSeconds / 3600;
    return totalHours * (Math.PI / 12);
  }

function parseRightAscension(raString) {
let [hours, minutes, seconds] = raString.split(':').map(Number);
return { hours, minutes, seconds };
}

function declinationToRadians(declinationString) {
    let [degrees, minutes, seconds] = declinationString.split(':').map(Number);
    degrees = Math.abs(degrees) * (degrees < 0 ? -1 : 1);
    const totalSeconds = 3600 * Math.abs(degrees) + 60 * minutes + seconds;
    const totalDegrees = totalSeconds / 3600;
    return totalDegrees * (Math.PI / 180) * (degrees < 0 ? -1 : 1);
}


// Here is a JavaScript function that maps the apparent magnitude of a star, ranges from -2 to 7, into values from 1 to 10, where 10 corresponds to the brightest star (-2 magnitude) and 1 corresponds to the dimmest star (7 magnitude)
function mapMagnitude(magnitude) {
        return 11 - (magnitude+3);
  }

writeFile('./public/files/stars_clean.json', JSON.stringify(stars_clean), function (err) { if (err) console.log(err);})
console.log('File saved to stars_clean.json')

// Spectral class to RGB value
// http://www.vendian.org/mncharity/dir3/starcolor/
// O     155 176 255  #9bb0ff  
// B     170 191 255  #aabfff  
// A     202 215 255  #cad7ff  
// F     248 247 255  #f8f7ff  
// G     255 244 234  #fff4ea  
// K     255 210 161  #ffd2a1  
// M     255 204 111  #ffcc6f 
function spectralClassToRGB(spectralClass) {
    switch (spectralClass) {
      case "O":
        return { r: 155, g: 176, b: 255 };
      case "B":
        return { r: 170, g: 191, b: 255 };
      case "A":
        return { r: 202, g: 215, b: 255 };
      case "F":
        return { r: 248, g: 247, b: 255 };
      case "G":
        return { r: 255, g: 244, b: 234 };
      case "K":
        return { r: 255, g: 210, b: 161 };
      case "M":
        return { r: 255, g: 204, b: 211 };
      default:
        return { r: 255, g: 255, b: 255 };
    }
  }