export function getNodes(labs, distance) {
    // By ChatGPT
    // This function uses a distance formula to determine if two latitude/longitude points are close to each other. If they are within a specified distance, they are added to the same group. If the next point is too far away, the current group is processed and the centroid of the group is added as a node. This process continues until all the labs have been processed and the final list of nodes is returned.
    let nodes = [];
    let group = [];
    
    labs.forEach((lab) => {
      if (group.length === 0) {
        group.push(lab);
      } else {
        let lastLab = group[group.length - 1];
        let latDiff = Math.abs(lab.latitude - lastLab.latitude);
        let longDiff = Math.abs(lab.longitude - lastLab.longitude);
        let totalDiff = Math.sqrt(Math.pow(latDiff, 2) + Math.pow(longDiff, 2));
  
        if (totalDiff > distance) {
          nodes.push(getCentroid(group));
          group = [];
        }
        group.push(lab);
      }
    });
    if (group.length > 0) {
      nodes.push(getCentroid(group));
    }
    return nodes;
  }
  
  export function getCentroid(group) {
    // By ChatGPT
    let sumLat = 0;
    let sumLong = 0;
    group.forEach((lab) => {
      sumLat += lab.latitude;
      sumLong += lab.longitude;
    });
    let centroid = {
      latitude: sumLat / group.length,
      longitude: sumLong / group.length,
    };
    return centroid;
  }
  
  export function getSunCoords() {
    // real sun position
    let datenow = new Date()
    let hour = datenow.getUTCHours() + datenow.getUTCMinutes()/60
    let day = datenow.getUTCDate() + datenow.getUTCMonth()*30.4  // number of days since start of the year
    let angle = 2*Math.PI*hour/24  // angle of sun as seen from north star when meridian 0 is at south
    let declination = ((23.44)*Math.sin(2*Math.PI*(day-79)/365.25))*2*Math.PI/180  // declination of sun in radians
    let distance = 1200
    let xsun = distance*Math.sin(angle)*Math.cos(declination)
    let zsun = -distance*Math.cos(angle)*Math.cos(declination)
    let ysun = Math.sin(declination)
    return [xsun, ysun, zsun];
  }
  